import logging
import json

from sitecomber.apps.shared.interfaces import BaseSiteTest

from .utils.article import is_reader_view_enabled, contains_placeholder_text, get_article_readtime
from .utils.spelling import check_spelling

logger = logging.getLogger('django')


def should_test_page(page):
    if not page.latest_request:
        return False
    if not page.is_internal:
        return False
    if not page.latest_request.response:
        return False
    if 'text/html' not in page.latest_request.response.content_type.lower():
        return False
    return True


class ReaderViewTest(BaseSiteTest):
    """
    Determins if the page has a structured article
    Uses library https://github.com/codelucas/newspaper/
    """

    def get_description_html(self):

        return """<p>Determins if the page has a structured article (with a title, body text,
    author, main image and publication date) and is therefore optimized for
    reader viewers such as Pocket, Instapaper or browser reader view.</p>
    <p>This test uses the Python library <a href="https://github.com/codelucas/newspaper/">Newspaper3k</a>
    to extract the article content.</p>
        """

    def on_page_parsed(self, page):
        from sitecomber.apps.results.models import PageTestResult

        if should_test_page(page):

            reader_view_enabled, status, message, data = is_reader_view_enabled(page, self.settings)

            r, created = PageTestResult.objects.get_or_create(
                page=page,
                test=self.class_path
            )
            r.message = message
            r.status = status
            try:
                r.data = json.dumps(data, sort_keys=True, indent=2)
            except Exception as e:
                logger.error(u"Error dumping JSON data: %s: %s" % (data, e))
            r.save()


class PlaceholderTextTest(BaseSiteTest):
    """
    This test looks for lorem or ipsum or tk in main article body and title
    """

    @property
    def placeholder_words(self):
        return ['lorem', 'ipsum'] if 'placeholder_words' not in self.settings else self.settings['placeholder_words']

    def get_description_html(self):

        return """<p>This test looks for the placeholder words \"%s\" in the main article body or title. </p>
            <small>To adjust which words are searched for, go to the Site configuration
            settings and scroll down to the "PlaceholderTextTest" section.
            Custom words should be entered as a JSON list, in this format:
            <pre>
              {
                "placeholder_words": [
                  "lorem",
                  "todo",
                  "tbd",
                ]
              }
            </pre></small></p>
        """ % ('", "'.join(self.placeholder_words))

    def on_page_parsed(self, page):
        from sitecomber.apps.results.models import PageTestResult

        if should_test_page(page):

            placeholder_text, message, data = contains_placeholder_text(page, self.settings, self.placeholder_words)
            status = PageTestResult.STATUS_SUCCESS if not placeholder_text else PageTestResult.STATUS_ERROR

            r, created = PageTestResult.objects.get_or_create(
                page=page,
                test=self.class_path
            )
            r.message = message
            r.status = status
            try:
                r.data = json.dumps(data, sort_keys=True, indent=2)
            except Exception as e:
                logger.error(u"Error dumping JSON data: %s: %s" % (data, e))
            r.save()


class ArticleReadTimeInfo(BaseSiteTest):
    """
    Determines approximate read time based on library
    https://github.com/alanhamlett/readtime
    """

    def get_description_html(self):

        return """<p>This test returns approximate read time based on <a href="https://help.medium.com/hc/en-us/articles/214991667-Read-time">Medium's read time forumula</a> of roughly 265 WPM.</p>
    <p>See the <a href="https://github.com/alanhamlett/readtime">readtime library</a> for more details.</p>
        """

    def on_page_parsed(self, page):
        from sitecomber.apps.results.models import PageTestResult

        if should_test_page(page):

            message, data = get_article_readtime(page, self.settings)

            r, created = PageTestResult.objects.get_or_create(
                page=page,
                test=self.class_path
            )
            r.message = message
            r.status = PageTestResult.STATUS_INFO
            try:
                r.data = json.dumps(data, sort_keys=True, indent=2)
            except Exception as e:
                logger.error(u"Error dumping JSON data: %s: %s" % (data, e))
            r.save()


class SpellCheckTest(BaseSiteTest):
    """
    Check spelling using pyspellchecker
    See https://github.com/barrust/pyspellchecker
    """

    def get_description_html(self):

        return """
            <p>This test checks spelling in the main article title and body text.</p>

            <p><small>If a word is caught that shouldn't be, it may be that this
            word is simply not in the Python NLTK corpus dictionary.</small></p>

            <p><small>To add a word to the custom dictionary, go to the Site
            configuration settings and scroll down to the "SpellCheckTest"
            section. Custom words should be entered as a JSON list, in this format:
            <pre>
              {
                "known_words": [
                  "swole",
                  "cheesemonger",
                  "gadzooks",
                ]
              }
            </pre></small></p>
        """

    def on_page_parsed(self, page):
        from sitecomber.apps.results.models import PageTestResult
        if should_test_page(page):

            contains_misspellings, message, data = check_spelling(page, self.settings)
            status = PageTestResult.STATUS_SUCCESS if not contains_misspellings else PageTestResult.STATUS_ERROR

            r, created = PageTestResult.objects.get_or_create(
                page=page,
                test=self.class_path
            )
            r.message = message
            r.status = status
            try:
                r.data = json.dumps(data, sort_keys=True, indent=2)
            except Exception as e:
                logger.error(u"Error dumping JSON data: %s: %s" % (data, e))
            r.save()



import re
import logging

from newspaper import Article
from newspaper.utils import get_available_languages

import readtime


logger = logging.getLogger('django')


def get_article(page, settings):
    url = page.url
    html = page.last_text_content

    language = 'en' if 'lang' not in settings else settings['lang']
    if language not in get_available_languages():
        logger.error(u"Language %s not found. Defaulting to 'en' instead." % (language))
        language = 'en'

    article = Article(url=url, language=language)
    article.download(html)
    article.parse()
    return article


def is_reader_view_enabled(page, settings):

    article = get_article(page, settings)

    reader_view_enabled = False
    messages = []
    data = {}
    if article.text and article.title:
        status = "success"
        reader_view_enabled = True
        messages.append(u"Page contains properly structured article.")

        if not article.top_image:
            messages.append(u"WARNING: Aricle missing top image.")
            status = "warning"

        if not article.authors:
            messages.append(u"WARNING: Aricle missing authors.")
            status = "warning"

        if not article.publish_date:
            messages.append(u"WARNING: Aricle missing publish date.")
            status = "warning"

        data = {
            'article': {
                'text': article.text,
                'title': article.title,
                'authors': article.authors,
                'publish_date': None if not article.publish_date else article.publish_date.isoformat(),
                'top_image': article.top_image,
                'imgs': list(article.imgs)
            }
        }
    else:
        messages.append(u"Page missing a structured article.")
        status = "error"

    message = u" ".join(messages)
    return reader_view_enabled, status, message, data


def get_placeholder_words(input_text, placeholder_words):

    placeholder_words_found = []
    for placeholder_string in placeholder_words:
        matches = re.finditer(r'\b%s\b' % (placeholder_string), input_text, re.IGNORECASE)
        for match in matches:
            placeholder_words_found.append(match.group(0))

    return placeholder_words_found


def contains_placeholder_text(page, settings, placeholder_words):
    article = get_article(page, settings)

    data = {'placeholder_words_searched': placeholder_words, 'placeholder_words_found': []}
    placeholder_words_found = get_placeholder_words(u"%s %s" % (article.title, article.text), placeholder_words)

    data['placeholder_words_found'] = placeholder_words_found
    if len(placeholder_words_found) > 0:
        return True, u"Found placeholder word(s): \"%s\"" % ('", "'.join(placeholder_words_found)), data
    else:
        message = 'No placeholder text "%s" found.' % ('", "'.join(placeholder_words))
        return False, message, data


def get_article_readtime(page, settings):

    article = get_article(page, settings)
    if article.text:
        result = readtime.of_text(article.text)
        return str(result.text), {'read_time': str(result.text)}

    return 'No article found', {}



from nltk.corpus import words

valid_one_letter_words = ['a', 'i', 'o']


def get_extended_dictionary():
    modern_technical_terminology = [
        "blog",
        "blogger",
        "website",
        "webpage",
        "homepage",
        "inbox",
        "inboxes",
        "hashtag",
        "email",
        "outsource",
        "timeline",
        "airfare",
        "paycheck",
        "whiteboard",
        "podcast",
        "livestream",
        "smartphone",
        "upload",
        "download",
        "listserv",
        "screensaver",
        "telecommute",
        "multitask",
        "elife",
        "infographic",
        "uppercase",
        "lowercase",
        "trademark",
        "playlist",
        "emoji",
        "earbud",
        "audiobook",
        "skillset",
        "upzip",

        "ipad",
        "iphone",
        "google",
        "ebay",
        "itunes",
        "instagram",
        "facebook",
        "twitter",
        "istock",
        "workweek"
    ]
    modern_social_terminology = [
        "who'd",
        "mom",
        "dad",
        "grandkid",
        "timeout",
        "bandana",
        "transgender",
        "cisgender",
        "heteronormative",
        "transvaginal",
        "descrimination",
        "anxious",
        "handwash",
        "restroom",
        "sanitizer",
        "misappropriate",
        "dystopian",
        "scrunchy",
        "rehab",
        "veggie",
        "prep",
        "indoctrinate"
        "attune",
        "upcycling",
        "sedate",
        "badass",
        "pushback",
        "inundate",
        "overshare",
        "cilantro",
        "reframing",
        "midcentury",
        "flatline",
        "weeknight",
        "prenup",
        "multigenerational",
        "microfiber",
        "agent",
        "caregiver",
        "pushup",
        "moisturizer",
        "bedsheet",
        "spritz",
        "photojournalist",
        "allergy",
        "granola",
        "hotcakes",
        "breadsticks",
        "counterintuitive",
        "meth",
        "foodie",
        "takeout",
        "kombucha",
        "skydive",
        "goop",
        "groundbreaking",
        "spandex",
        "cardio",
        "popsicle",
        "walkup",
        "townhouse",
        "condo",
        "zit",
        "eyeroll",
        "pilates",
        "ultrasound",
        "cesarean",
        "sweatpants",
        "paralegal",
        "gatekeep",
        "meetup",
        "waitstaff",
        "clutter",
        "zipper",
        "deadbolt",
        "hoagie",
        "surefooted",
        "racecar",
        "smurf",
        "humongous",
        "scattershot",
        "debuting",
        "vegan",
        "yearslong",
        'keychain',
        'laundromat',
        'antsy',
        'foodborne',
        'blindside',
        'sheetcake',
        'meh',
        'sleepover'
    ]

    alternate_spellings = [
        'catalog'
    ]

    # TODO -- flesh out loan words more completely
    adopted_words = [
        "reiki",
        'mâché',
        'guac',
        'queso',
        'nacho',
        'quesadilla',
        'mitzvah',
        'naïve',
        'fiancée',
        'à',
        'latte',
        'blasé',
        'purée',
        'tchotchke',
        'décor',
        'bébé',
        'cliché',
        'doppelgänger',
        'kitschy',
        'risqué',
        'schlep',
        'pâtés',
        'piñata',
        'lasagna',
        'quinceañera',
        'ganache',
        'carpaccio',
        'passé',
        'macadamia',
        'rosé',
        'bratwurst'

    ]

    return set(words.words() + modern_technical_terminology + modern_social_terminology + alternate_spellings + adopted_words)



import logging
import re
from string import punctuation
import unicodedata

from nltk.corpus import stopwords

import contractions

from spellchecker import SpellChecker

from .article import get_article
from .dictionary import get_extended_dictionary


logger = logging.getLogger('django')


def check_spelling(page, settings):
    # Validate that this is for English; currently only English is supported
    language = 'en' if 'lang' not in settings else settings['lang']

    article = get_article(page, settings)
    custom_known_words = [] if 'known_words' not in settings else settings['known_words']

    dictionary = set(list(get_extended_dictionary()) + list(custom_known_words))

    if article.text:
        raw_text = u'%s. %s' % (article.title, article.text)
        misspelled = get_misspelled_words(raw_text, language, dictionary)
        found_misspellings = len(misspelled) > 0
        message = "No misspellings found" if not found_misspellings else u'Found %s misspelling(s): "%s"' % (len(misspelled), '", "'.join(misspelled))
        return found_misspellings, message, {'misspelled_words': misspelled}

    return False, 'No article found', {}


def is_in_dictionary(word, dictionary):
    if len(word) == 1:
        if word.lower() == 'a' or word.lower() == 'i' or word.lower() == 'o':
            return True
    else:
        return (word in dictionary)


def get_simplification_options(word):
    suffixes = [
        {'able': ''},
        {'acy': ''},
        {'al': ''},
        {'ance': ''},
        {'ate': ''},
        {'bed': ''},
        {'bility': ''},
        {'bility': 'ble'},
        {'bio': ''},
        {'dom': ''},
        {'cced': 'c'},
        {'ccing': 'c'},
        {'dded': 'd'},
        {'dding': 'd'},
        {'ed': ''},
        {'ed': 'e'},
        {'ee': ''},
        {'en': ''},
        {'en': 'e'},
        {'ence': ''},
        {'ence': 'e'},
        {'er': ''},
        {'er': 'e'},
        {'erizer': ''},
        {'es': ''},
        {'es': 'e'},
        {'esque': ''},
        {'est': ''},
        {'ffed': 'f'},
        {'ffing': 'f'},
        {'ful': ''},
        {'fy': ''},
        {'gged': 'g'},
        {'gging': 'g'},
        {'hood': ''},
        {'ible': ''},
        {'ic': ''},
        {'ical': ''},
        {'ied': ''},
        {'ied': 'y'},
        {'ier': ''},
        {'ier': 'y'},
        {'ies': ''},
        {'ies': 'y'},
        {'iest': ''},
        {'iest': 'y'},
        {'ify': ''},
        {'ily': ''},
        {'iness': ''},
        {'iness': 'y'},
        {'ing': ''},
        {'ing': 'e'},
        {'ious': ''},
        {'ise': ''},
        {'ish': ''},
        {'ism': ''},
        {'ist': ''},
        {'ity': ''},
        {'ity': 'y'},
        {'ive': ''},
        {'ize': ''},
        {'izer': ''},
        {'jjed': 'j'},
        {'jjing': 'j'},
        {'kked': 'k'},
        {'kking': 'k'},
        {'less': ''},
        {'like': ''},
        {'lled': 'l'},
        {'lling': 'l'},
        {'long': ''},
        {'ly': ''},
        {'mate': ''},
        {'ment': ''},
        {'mmed': 'm'},
        {'mming': 'm'},
        {'ness': ''},
        {'nned': 'n'},
        {'nning': 'n'},
        {'ologist': ''},
        {'ologist': 'ology'},
        {'ous': ''},
        {'ped': ''},
        {'pped': 'p'},
        {'pping': 'p'},
        {'qqed': 'q'},
        {'qqing': 'q'},
        {'red': ''},
        {'red': 're'},
        {'rred': 'r'},
        {'rring': 'r'},
        {'s': ''},
        {'sion': ''},
        {'ssed': 's'},
        {'ssing': 's'},
        {'tion': ''},
        {'tion': 'te'},
        {'tize': ''},
        {'tize': 'ty'},
        {'tize': 't'},
        {'tted': 't'},
        {'tting': 't'},
        {'ty': ''},
        {'vved': 'v'},
        {'vving': 'v'},
        {'ward': ''},
        {'wards': ''},
        {'wide': ''},
        {'wise': ''},
        {'worthy': ''},
        {'y': ''}
    ]

    prefixes = [
        {'ante': ''},
        {'anti': ''},
        {'auto': ''},
        {'bi': ''},
        {'bio': ''},
        {'bis': ''},
        {'co': ''},
        {'de': ''},
        {'dis': ''},
        {'en': ''},
        {'ex': ''},
        {'extra': ''},
        {'hyper': ''},
        {'ig': ''},
        {'im': ''},
        {'in': ''},
        {'inter': ''},
        {'ir': ''},
        {'macro': ''},
        {'mal': ''},
        {'mega': ''},
        {'micro': ''},
        {'mini': ''},
        {'mis': ''},
        {'mono': ''},
        {'multi': ''},
        {'neo': ''},
        {'neuro': ''},
        {'non': ''},
        {'omni': ''},
        {'over': ''},
        {'penta': ''},
        {'per': ''},
        {'poly': ''},
        {'post': ''},
        {'pre': ''},
        {'pro': ''},
        {'quad': ''},
        {'re': ''},
        {'retro': ''},
        {'semi': ''},
        {'socio': ''},
        {'sub': ''},
        {'super': ''},
        {'tran': ''},
        {'tri': ''},
        {'un': ''},
        {'under': ''},
        {'uni': ''}
    ]
    # Sort prefixes and suffixes from longest to shortest
    suffixes.sort(key=lambda s: len(next(iter(s))))
    suffixes.reverse()
    prefixes.sort(key=lambda s: len(next(iter(s))))
    prefixes.reverse()
    output = []

    for prefix_item in prefixes:
        prefix = next(iter(prefix_item))
        if word.startswith(prefix):
            output.append({
                'type': 'prefix',
                'search': prefix,
                'replace': prefix_item[prefix]
            })

    for suffix_item in suffixes:
        suffix = next(iter(suffix_item))
        if word.endswith(suffix):
            output.append({
                'type': 'suffix',
                'search': suffix,
                'replace': suffix_item[suffix]
            })

    return output


def apply_simplification(word, simplification):
    if simplification['type'] == 'prefix':
        if word.startswith(simplification['search']):
            word = simplification['replace'] + word[len(simplification['search']):]

    if simplification['type'] == 'suffix':
        if word.endswith(simplification['search']):
            word = word[:-len(simplification['search'])] + simplification['replace']

    return word


def simplify_word(word, dictionary, debug=False):
    log_level = logging.WARNING if debug else logging.DEBUG

    logger.log(log_level, u"\n--------- Simplifying %s ---------" % (word))

    possible_simplifications = get_simplification_options(word)
    logger.log(log_level, "Possible simplifications: %s " % (possible_simplifications))

    if len(possible_simplifications) == 0:
        logger.log(log_level, "No more simplification options found, returning %s" % (word))
        return word

    for simplification in possible_simplifications:

        applied = apply_simplification(word, simplification)
        logger.log(log_level, "Applied simplification %s replaced --> %s" % (simplification, applied))
        if is_in_dictionary(applied, dictionary):
            logger.log(log_level, "Simplification yielded valid word %s" % (applied))
            return applied

        else:
            drilled_down = simplify_word(applied, dictionary, debug)
            if is_in_dictionary(drilled_down, dictionary):
                logger.log(log_level, "Drilled down yielded valid word %s" % (drilled_down))
                return drilled_down
    return word


def remove_emails(input):
    return re.sub(r"\S*@\S*\s?", " ", input)


def remove_hashes(input):
    return re.sub(r"#(\w+)", " ", input)


def remove_phonenumbers(input):
    # TODO
    # intl_removed = re.sub(r'(\+[0-9]+\s*)?(\([0-9]+\))?[\s0-9\-]+[0-9]+', ' ', input)
    # intl_removed = input
    intl_removed = re.sub(r"(\d{1,3}[-\.\s]??\d{3}[-\.\s]??\d{3}[-\.\s]??\d{4}|\(\d{3}\)\s*\d{3}[-\.\s]??\d{4}|\d{3}[-\.\s]??\d{4})", " ", input)
    us_removed = re.sub(r"(\d{1,3}[-\.\s]??\d{3}[-\.\s]??\d{3}[-\.\s]??\d{4}|\(\d{3}\)\s*\d{3}[-\.\s]??\d{4}|\d{3}[-\.\s]??\d{4})", " ", intl_removed)

    return us_removed


def remove_urls(input):
    # return re.sub(r'\s*(?:https?://)?\S*\.[A-Za-z]{2,5}\s*', " ", input)
    removed_full_links = re.sub(r'(http|https|ftp|telnet):\/\/[\w\-_]+(\.[\w\-_]+)+([\w\-\.,@?^=%&amp;:/~\+#]*[\w\-\@?^=%&amp;/~\+#])?', " ", input)
    remove_partial_links = re.sub(r"([\w\.]+\.(?:com|org|net|us|co|edu|gov|uk)[^,\s]*)", " ", removed_full_links)
    remove_mailtos = re.sub(r'((mailto\:|(news|(ht|f)tp(s?))\://){1}\S+)', " ", remove_partial_links)
    ips_removed = re.sub(r"\b\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\b", " ", remove_mailtos)
    intl_removed = re.sub(r'(tel):(\+[0-9]+\s*)?(\([0-9]+\))?[\s0-9\-]+[0-9]+', ' ', ips_removed)
    us_removed = re.sub(r"(tel):(\d{3}[-\.\s]??\d{3}[-\.\s]??\d{4}|\(\d{3}\)\s*\d{3}[-\.\s]??\d{4}|\d{3}[-\.\s]??\d{4})", " ", intl_removed)
    filenames_removed = re.sub(r"([\w\d\-.]+\.(pdf|PDF|doc|DOC|docx|DOCX|zip|ZIP|xlsx|XLSX|csv|CSV))", " ", us_removed)
    return filenames_removed


def remove_acronyms(input):
    return re.sub(r"\b[A-Z\.]{2,}s?\b", "", input)


def remove_direct_quotation_brackets(input):
    return input.replace("[", "").replace("]", "")


def get_misspelled_words(raw_text, language, dictionary, debug=False):
    log_level = logging.WARNING if debug else logging.DEBUG

    # if language != 'en':
    #     return True, 'Language "%s" not supported' % (language)
    spell = SpellChecker(language=language, distance=1)

    logger.log(log_level, ">> raw_text:")
    logger.log(log_level, raw_text)

    # Remove email addresses, hashes, urls, phone numbers...
    urls_removed = remove_urls(raw_text)
    emails_removed = remove_emails(urls_removed)
    hashes_removed = remove_hashes(emails_removed)
    phonenumbers_removed = remove_phonenumbers(hashes_removed)

    logger.log(log_level, ">> after email, hashes, urls, phone numbers removed:")
    logger.log(log_level, phonenumbers_removed)

    # Replace fancy typigraphic characters like curly quotes and em dashes
    typographic_translation_table = dict([(ord(x), ord(y)) for x, y in zip(u"‘’´'“”–-—⁃‐…●•∙©", u"''''\"\"-----.----")])
    typography_removed = phonenumbers_removed.translate(typographic_translation_table)
    hyphens_removed = typography_removed.replace("-", " ").replace("/", " ")
    newlines_removed = hyphens_removed.replace("\n", " ").replace("\r", " ")

    logger.log(log_level, ">> after fancy typographic characters and newlines removed:")
    logger.log(log_level, newlines_removed)

    contractions_removed = contractions.fix(newlines_removed)
    possessives_removed = re.sub("\'s ", " ", contractions_removed)
    hyphens_removed = possessives_removed.replace("-", " ")
    acronyms_removed = remove_acronyms(hyphens_removed)
    whitespace_condensed = re.sub("[ \t]+", " ", acronyms_removed.replace(u'\u200b', ' '))

    logger.log(log_level, ">> after contractions, posessives, hyphens and acronyms removed:")
    logger.log(log_level, whitespace_condensed)

    # Split text into words
    check_words_raw = whitespace_condensed.split(' ')
    logger.log(log_level, ">> check_words_raw:")
    logger.log(log_level, check_words_raw)

    # Remove stopwords for faster processing
    stop_words = set(stopwords.words('english'))
    stopwords_removed = [word for word in check_words_raw if (word.lower() not in stop_words)]
    logger.log(log_level, ">> stopwords_removed:")
    logger.log(log_level, stopwords_removed)

    # Remove any numbers and punctuation
    normalzized_words = [unicodedata.normalize('NFKC', word) for word in stopwords_removed]
    punctuation_removed = [remove_direct_quotation_brackets(word.strip(punctuation)) for word in normalzized_words if (word and not word[0].isdigit())]
    # Apply twice in case there is punctuation around digits
    punctuation_removed = [remove_direct_quotation_brackets(word.strip(punctuation)) for word in punctuation_removed if (word and not word[0].isdigit())]

    logger.log(log_level, ">> punctuation_removed:")
    logger.log(log_level, punctuation_removed)

    remove_empty_words = [word for word in punctuation_removed if word]

    # Gather list of assumed proper nouns.
    # Assume anything capitalized in article is a local proper noun
    proper_nouns = []
    for word in remove_empty_words:
        if word[0].isupper() and not is_in_dictionary(simplify_word(word.lower(), dictionary), dictionary):
            proper_nouns.append(word.strip(punctuation))
    proper_nouns_lower = [word.lower() for word in proper_nouns]
    logger.log(log_level, ">> proper_nouns:")
    logger.log(log_level, proper_nouns)

    # Remove anything matching a proper noun from above
    remove_proper_nounds = [item for item in remove_empty_words if item.lower() not in proper_nouns_lower]

    # Reduce to unique set of words
    check_words = list(set(remove_proper_nounds))
    logger.log(log_level, ">> check_words:")
    logger.log(log_level, check_words)

    # First check the corpus dictionary:
    words_not_in_dict = [word for word in check_words if not is_in_dictionary(word.lower(), dictionary)]
    logger.log(log_level, ">> words_not_in_dict:")
    logger.log(log_level, words_not_in_dict)

    # Next use spelling library
    unknown = [item for item in list(spell.unknown(words_not_in_dict))]
    logger.log(log_level, ">> unknown:")
    logger.log(log_level, unknown)

    # Finally, removing prefix and suffixes to unearth a valid root word
    misspelled = []
    for word in unknown:
        simplified_word = simplify_word(word, dictionary)
        if not is_in_dictionary(simplified_word, dictionary):
            misspelled.append(simplified_word)

    logger.log(log_level, ">> misspelled:")
    logger.log(log_level, misspelled)

    return misspelled



from ..utils.spelling import remove_emails, remove_hashes, remove_phonenumbers, \
    remove_urls, get_misspelled_words, simplify_word
from ..utils.dictionary import get_extended_dictionary


def test():
    print("Testing spelling...")

    email_input = u"Email test: abc user@xxx.com 123 any@www foo @ bar 78@ppp @5555 aa@111 testing @MyFancyHandle"
    email_expected_output = u"Email test: abc  123  foo  bar    testing  "
    email_actual_output = remove_emails(email_input)
    if email_expected_output != email_actual_output:
        raise Exception("Email removing function returned unexpected output. \nExpected '%s' \nReceieved '%s' " % (email_expected_output, email_actual_output))

    hash_input = u"Hash tag test: this sentence has #fancyhashtag and #MyTag and #mytag# #111Tag"
    hash_expected_output = u"Hash tag test: this sentence has   and   and  #  "
    hash_actual_output = remove_hashes(hash_input)
    if hash_expected_output != hash_actual_output:
        raise Exception("Hash removing function returned unexpected output. \nExpected '%s' \nReceieved '%s' " % (hash_expected_output, hash_actual_output))
    else:
        print("Hash removing logic successful")

    # TODO: 1-800-ALPHNUM  1.222.333.1234 | 1-223-123-1232 | 12223334444 1-(123)-123-1234 | 123 123 1234
    phone_input = u"Phone number test: 1-800-222-3456 and 1-800-abc-defg and 123-456-7890 and (123)456-7890 and +1-816-555-1212 and 1234567890 and asdl238()"
    phone_expected_output = u'Phone number test:   and 1-800-abc-defg and 123-  and   and +  and  890 and asdl238()'
    # TODO -- better phone number detection
    # u"Phone number test: and   and   and   and   and   and asdl238()"
    phone_actual_output = remove_phonenumbers(phone_input)
    if phone_expected_output != phone_actual_output:
        raise Exception("Phone number removing function returned unexpected output. \nInput '%s' \nExpected '%s' \nReceieved '%s' " % (phone_input, phone_expected_output, phone_actual_output))
    else:
        print("Phone number removing logic successful")

    url_input = u"""
Missing space between this sentence.Another the next one.
URL Test: http://www.example1.com example2.com https://example3.com
Words separated by slashes: and/or happy/sad/etc
Filename Test: .PNG or .JPG format, myfile.pdf or.other filename.doc
URL: ftp://ftp.is.co.za/rfc/rfc1808.txt test text
URL: http://www.ietf.org/rfc/rfc2396.txt test text
URL: mailto:John.Doe@example.com test text
URL: telnet://192.0.2.16:80/ test text
URN (not URL): urn:oasis:names:specification:docbook:dtd:xml:4.1.2 test text
URN (not URL): tel:+1-816-555-1212 tel:1234567890 (?)
IP: 192.0.2.16
URL with query: http://regexlib.com/REDetails.aspx?regexp_id=x#Details
"""
    url_expected_output = u"""
Missing space between this sentence.Another the next one.
URL Test:
Words separated by slashes: and/or happy/sad/etc
Filename Test: .PNG or .JPG format,   or.other
URL:   test text
URL:   test text
URL:    test text
URL:   test text
URN (not URL): urn:oasis:names:specification:docbook:dtd:xml:4.1.2 test text
URN (not URL):     (?)
IP:
URL with query:
"""
    url_actual_output = remove_urls(url_input)
    if url_expected_output != url_actual_output:
        # TODO -- string comparison not working
        # raise Exception("URL removing function returned unexpected output. \nExpected '%s' \nReceieved '%s' " % (url_expected_output, url_actual_output))
        print("URL removing function returned unexpected output. \nExpected '%s' \nReceieved '%s' " % (url_expected_output, url_actual_output))
    else:
        print("URL removing logic successful")

    # emove_emails, remove_hashes, remove_phonenumbers, \
    # remove_urls, remove_acronyms,

    words_to_simplify = ['mustardy', 'misstated', 'rashy', 'rashiness',
                         'toddlerhood', 'clinginess', 'seatmates',
                         'grandparenting', 'moisterizers', 'recertified',
                         'cringeworthy', 'reoccurred', 'misappropriating',
                         'handwashing', 'declutter', 'unshowered',
                         'underappreciated', 'reconfigured', 'cataloging',
                         'attendee', 'nonjudgmentally', 'tearily', 'destigmatize',
                         'prepped', 'fibbed', 'tantrumming']
    expected_output = ['mustard', 'stated', 'rash', 'rash', 'toddler',
                       'clingy', 'seat', 'grandparent', 'moist', 'certified',
                       'cringe', 'occur', 'appropriate', 'handwash',
                       'clutter', 'shower', 'appreciate',
                       'configure', 'catalog', 'attend', 'judgment', 'tear', 'stigmatize',
                       'prep', 'fib', 'tantrum']

    # words_to_simplify = ['tantrumming']
    # expected_output = ['tantrum']
    extended_dictionary = get_extended_dictionary()

    output = []
    for word in words_to_simplify:
        simplified = simplify_word(word, extended_dictionary, False)
        if simplified not in extended_dictionary:
            raise Exception("Word %s simplified from %s didn't yield a valid simplification" % (word, simplified))
        # print("From %s -> %s In dictionary? %s" % (word, simplified, (simplified in extended_dictionary)))
        output.append(simplified)

    if output != expected_output:
        raise Exception("Words incorrectly simplified \nFrom:\n%s \nTo:\n%s \nExpected:\n%s" % (words_to_simplify, output, expected_output))
    print("Simplification test successful")

    test_text = u"""
This is an example with proper nouns Dr. Marelle Jazdin, a doctor. Marelle Jazdin is a doctor. Jazdin thinks you should be mindful.
Acronym test: ABCD, A.B.C.D, and even A.B.C.Ds.
Fancy typography test: This is a sentence with fancy curly quotes,” it’s great to have fancy typography like ‘single quotes’ ´back ticks various dashes –-—⁃ ellipses … and fancy bullets •∙ ©
Direct Quote test: she was "fall[ing] down the stairs" “It [driving] imposes a heavy procedural workload on cognition that . . . leaves little processing capacity available for other tasks” (Salvucci and Taatgen 107). [1] Salvucci and Taatgen propose that “[t]he heavy cognitive workload of driving suggests that any secondary task has the potential to affect driver behavior” (108).

What's up with these apostrophes? half's baby's today's

Email test: abc user@xxx.com 123 any@www foo @ bar 78@ppp @5555 aa@111 testing @MyFancyHandle

Hash tag test: this sentence has #fancyhashtag and #MyTag and #mytag# #111Tag

Phone number test: 1-800-222-3456 and 123-456-7890 and +1-816-555-1212 (123)456-7890 1.222.333.1234 | 1-223-123-1232 | 12223334444

URL Test: http://www.example1.com example2.com https://example3.com
Words separated by slashes: and/or happy/sad/etc
Filename Test: .PNG or .JPG format, myfile.pdf filename.doc
URL: ftp://ftp.is.co.za/rfc/rfc1808.txt test text
URL: http://www.ietf.org/rfc/rfc2396.txt test text
URL: mailto:John.Doe@example.com test text
URL: telnet://192.0.2.16:80/ test text
URN (not URL): tel:+1-816-555-1212 (?)
IP: 192.0.2.16
URL with query: http://regexlib.com/REDetails.aspx?regexp_id=x#Details

Let's test numbers: 5,000 30,500 and 1,000,045
What's your favorite decade? The 50s, '60s, 70s, 1980s, 90s or something else? I personally like 1986.
Dates and Quantities: 10th 1st ½ 1/1/2000 123,456 $99.50

Proper noun test: with Gretta Byonce (Sheila Lee) and then "Ralecio Twahla" and

Compound words test: grandparenting, toddlerhood, seatmates, mustardy,
hallucinated, moisterizers, misstated, rashy, clinginess,
sanitizers handwashing recertified bandanas restrooms desensitized weepier
scrunchies misappropriating iphones gummy candy yearslong medicating flatline
telecommute emailed transvaginal commoditize counterintuitively pickiest
well‐to‐do breast‐feeding nonbinding veganism baby's underappreciated
trademarked hypercompetitive reconfigured dumbest adjustability unshowered
prepped women's declutter surefootedness cataloging e-commerce cannibalized
keychains nonjudgmentally tearily moms paycheck hashtag sleepover bandanas
restroom sanitizer handwash descrimination cartoonish workout

Loan Words (german) : angst blitz blitzkrieg bratwurst cobalt concertmaster
dachshund gauss deli delicatessen pinscher doppelgänger doppelganger doppler
ersatz fahrenheit fest flak flack frankfurter

TODO -- complete loan word list spanish, french, yiddish, etc
"""

    print("Getting misspelled words - round 1...")
    misspelled_words = get_misspelled_words(test_text, "en", extended_dictionary, False)
    if len(misspelled_words) > 0:
        raise Exception("Words incorrectly flagged. Should have found 0 misspellings, but instead found: %s" % (misspelled_words))

    print("Getting misspelled words - round 2...")
    # TODO -- differentiate between a valid numbery word (10th, 1/2, $99, etc) and an invalid numbery word (434kad4, etc)
    test_text_with_errors = u"""asdl2384() 29UDUDJS 1stish asdflkjd ½ 10th"""
    misspelled_words = get_misspelled_words(test_text_with_errors, "en", extended_dictionary, False)
    expected_error_count = 2
    if len(misspelled_words) != expected_error_count:
        raise Exception("Words incorrectly flagged. Should have found %s misspellings from %s but receieved: %s" % (expected_error_count, test_text_with_errors, misspelled_words))

    print("Done testing spelling!")



from ..utils.article import get_placeholder_words


def test():

    print("Test placeholder flagging...")
    placeholder_words = ['lorem', 'ipsum', 'tk', 'todo']
    input_text = u"This is an example sentance with lorem Notsum Lorem and TODO and now klorem bipsum batkite"
    expected_placeholder_words = ['lorem', 'Lorem', 'TODO']
    actual_placeholder_words = get_placeholder_words(input_text, placeholder_words)

    if expected_placeholder_words != actual_placeholder_words:
        raise Exception("Placeholder word finder got unexpected output. \nExpected '%s' \nReceieved '%s' " % (expected_placeholder_words, actual_placeholder_words))

    print("Done testing placeholder functions!")
